import typing

from django.conf import settings
from django.contrib.auth import get_user_model
from spadesdk.executor import Executor
from spadesdk.executor import Process as SDKProcess
from spadesdk.executor import RunResult as SDKRunResult
from spadesdk.history_provider import HistoryProvider

from ..utils.imports import import_object
from .models import Process, ProcessRun

User = get_user_model()


class ProcessService:
    @staticmethod
    def run_process(process: Process, user, user_params: dict) -> ProcessRun:
        """Run a process using the executor."""

        executor: Executor
        if (object_key := process.executor.callable) not in settings.SPADE_PROCESS_EXECUTORS:
            executor = import_object(object_key)
            settings.SPADE_PROCESS_EXECUTORS[object_key] = executor
        else:
            executor = settings.SPADE_PROCESS_EXECUTORS[object_key]

        run: ProcessRun = ProcessRun.objects.create(
            process=process,
            user=user,
            status=ProcessRun.Statuses.RUNNING,
            user_params=user_params,
        )

        try:
            result: SDKRunResult = executor.run(
                SDKProcess(
                    code=process.code,
                    system_params=process.system_params,
                ),
                user_params,
                user.id,
            )
            run.result = result.result.value if result.result else None
            run.output = result.output
            run.error_message = result.error_message
            run.status = ProcessRun.Statuses.FINISHED
            run.save()
        except Exception as e:
            run.status = ProcessRun.Statuses.ERROR
            run.result = ProcessRun.Results.FAILED
            run.error_message = str(e)
            run.save()

        return run

    @staticmethod
    def get_runs(process: Process, request, *args, **kwargs) -> typing.Iterable[ProcessRun]:
        """Get the runs for a process."""

        if not process.executor.history_provider_callable:
            return ProcessRun.objects.filter(process=process).order_by("-pk")

        history_provider: HistoryProvider
        if (object_key := process.executor.history_provider_callable) not in settings.SPADE_HISTORY_PROVIDERS:
            history_provider = import_object(object_key)
            settings.SPADE_HISTORY_PROVIDERS[object_key] = history_provider
        else:
            history_provider = settings.SPADE_HISTORY_PROVIDERS[object_key]

        provider_results = history_provider.get_runs(process, request, *args, **kwargs)
        return (
            ProcessRun(
                process=process,
                status=result.status.value,
                result=result.result.value if result.result else None,
                output=result.output,
                error_message=result.error_message,
                created_at=result.created_at,
                user=User.objects.get(id=result.user_id) if result.user_id else None,
            )
            for result in provider_results
        )
